var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var delay_exports = {};
__export(delay_exports, {
  default: () => delay_default
});
module.exports = __toCommonJS(delay_exports);
var import_constants = require("../constants");
class delay_default {
  constructor(plugin) {
    this.plugin = plugin;
    plugin.app.on("beforeStart", this.load);
    plugin.app.on("beforeStop", this.unload);
  }
  timers = /* @__PURE__ */ new Map();
  load = async () => {
    const { model } = this.plugin.db.getCollection("jobs");
    const jobs = await model.findAll({
      where: {
        status: import_constants.JOB_STATUS.PENDING
      },
      include: [
        {
          association: "execution",
          attributes: [],
          where: {
            status: import_constants.EXECUTION_STATUS.STARTED
          },
          required: true
        },
        {
          association: "node",
          attributes: ["config"],
          where: {
            type: "delay"
          },
          required: true
        }
      ]
    });
    jobs.forEach((job) => {
      this.schedule(job);
    });
  };
  unload = () => {
    for (const timer of this.timers.values()) {
      clearTimeout(timer);
    }
    this.timers = /* @__PURE__ */ new Map();
  };
  schedule(job) {
    const now = /* @__PURE__ */ new Date();
    const createdAt = Date.parse(job.createdAt);
    const delay = createdAt + job.node.config.duration - now.getTime();
    if (delay > 0) {
      const trigger = this.trigger.bind(this, job);
      this.timers.set(job.id, setTimeout(trigger, delay));
    } else {
      this.trigger(job);
    }
  }
  async trigger(job) {
    if (!job.execution) {
      job.execution = await job.getExecution();
    }
    if (job.execution.status === import_constants.EXECUTION_STATUS.STARTED) {
      this.plugin.resume(job);
    }
    if (this.timers.get(job.id)) {
      this.timers.delete(job.id);
    }
  }
  run = async (node, prevJob, processor) => {
    const job = await processor.saveJob({
      status: import_constants.JOB_STATUS.PENDING,
      result: null,
      nodeId: node.id,
      upstreamId: (prevJob == null ? void 0 : prevJob.id) ?? null
    });
    job.node = node;
    this.schedule(job);
    return processor.exit();
  };
  resume = async (node, prevJob, processor) => {
    const { endStatus } = node.config;
    prevJob.set("status", endStatus);
    return prevJob;
  };
}
